/*
 * PM - 2017
 * Obstacole Avoiding Robot
 * Chitu Maria Cristina 332CC
 */


#define F_CPU 16000000
/* am quart de 16MHz pe placa */

/* driver */
#define INPUT1 PC3
#define INPUT2 PC2
#define INPUT3 PC1
#define INPUT4 PC0

#define ENABLE_A PD5
#define ENABLE_B PD4

/* senzor */
#define ADC_INIT_CHANNEL 7
#define critical_DISTANCE 450

/* leduri */
#define LED_ROSU PD3
#define LED_VERDE PC4

#include <avr/io.h>
#include <util/delay.h>


/* functie de initializare a Timer1 */
void initialize_timer()
{
	/* folosim timer1 */
		
	/* NON-INVERTING MODE */
	TCCR1A |= (1 << COM1A1)  | (1 << COM1B1);
	
	/* FAST-PWM 8-bit */
	TCCR1A |= (1 << WGM10);
	TCCR1B |= (1 << WGM12);
	
	/* Setarea prescaler-ului */
	TCCR1B |= (1 << CS12);
	
	/* setam viteza initiala a fiecarui motor */
	OCR1A = 0; /* 0->255 */
	OCR1B = 0; /* 0->255 */
	
}

/* functie de initializare a ADC */
void initialize_ADC()
{
	/* am nevoie sa intializez canalul 7 */
	
	/* reference AVCC with external capacitor at AREF pin */
	ADMUX = (1 << REFS0);	

	/* ADEN este bitul de Enable si activeaza convertorul */
	ADCSRA = (1 << ADEN); 
	
	/*
	 ADPS2..0 selecteaza rata de esantionare pentru convertor
	 Aceasta poate fi de maxim fclk/2
	*/
	
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
	
}


/* functie ce returneaza rezultatul conversiei ADC 
	porneste conversia si asteapta rezultatul */
int ADC_get_conversion(uint8_t channel)
{
	
	/* porneste conversia ADC pe canalul "channel" */
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;
	ADCSRA = (1 << ADEN) | (1 << ADPS2);	/* e necesar ca sa termin conversia */
    ADCSRA |= (1 << ADSC);	/* porneste fiecare conversie */

	/* astept sa se termine conversia */
	loop_until_bit_is_set(ADCSRA, ADIF);
	    
	/* returneaza rezultatul obtinut */
    return ADC;
}


/* setez viteza standard a motoarelor */
void basic_speed()
{
	OCR1A = 100; 
	OCR1B = 100;
}

 /* se creste treptat viteza cand merge drept */
void increase_speed()
{
	if(OCR1A < 200 && OCR1B < 200)	{
			OCR1A += 10;
			OCR1B += 10;
	}
}


/* functie de initializare a motoarelor*/
void initialize_motors()
{
	/*
	 * PC3:0 sunt pinii de la uC care sunt folositi sa trimita la cele 4 intrari ale driver-ului
	 * sunt pini de iesire, informatia se duce catre driver
	 */
	
	/* motor 1 */
	DDRC |= (1 << INPUT1);
	DDRC |= (1 << INPUT2);
	
	/* 
	 *avem nevoie de enable ca sa permitem functionarea motorului
	 * avem 'si' logic in schma driverului, in care una dintre intrari o reprezinta enable
	 */
	DDRD |= (1 << ENABLE_A);
	
	/* motor 2 */
	DDRC |= (1 << INPUT3);
	DDRC |= (1 << INPUT4);
	DDRD |= (1 << ENABLE_B);
}



/* functie pentru inaintarea robotelului */
void go_forward()
{
	/* motor 1 */
	PORTC |= (1 << INPUT1);
	PORTC &= ~(1 << INPUT2);	/* Scriere bit pe 0 */
	
	/* motor 2 */
	PORTC &= ~(1 << INPUT3);	/* Scriere bit pe 0 */
	PORTC |= (1 << INPUT4);
	

}

/* functie folosita pentru ca robotelul sa mearga cu spatele */
void go_back()
{
	/* motor 1 */
	PORTC |= (1 << INPUT2);
	PORTC &= ~(1 << INPUT1);	/* Scriere bit pe 0 */
	
	/* motor 2 */
	PORTC |= (1 << INPUT3);
	PORTC &= ~(1 << INPUT4);	/* Scriere bit pe 0 */
	
	basic_speed();
}


/* functi ce seteaza 'vitezele' motoarelor la zero
 * oprirea acestuia
 */
void stop_motors()
{
	OCR1A = 0; 
	OCR1B = 0;
}


/* functie folosita pentru ca robotul sa vireze la dreapta */
void switch_right()
{
	/* motor 1 */
	/* se roteste in fata */
	PORTC |= (1 << INPUT1);
	PORTC &= ~(1 << INPUT2);
	
	/* motor 2 */
	/* se roteste in spate */
	PORTC |= (1 << INPUT3);
	PORTC &= ~(1 << INPUT4); 
	
	basic_speed();
	
	_delay_ms(350);
}



/* setez pinul pe care se vor primi date de la senzor */
void init_sensor()
{
	/* pin de intrare */
	DDRA  &= ~(1 << PA7);
}


/* initializare a pinilor pentru leduri */
void init_led()
{
	/* setez led-uri ca iesiri si stinse initial */
	DDRD |= (1 << LED_ROSU);
	PORTD &= ~(1 << LED_ROSU);
	
	DDRC |= (1 << LED_VERDE);
	PORTC &= ~(1 << LED_VERDE);
}


/* functii de alternare a culorilor celor doua leduri */

void change_to_green()
{
	PORTD &= ~(1 << LED_ROSU);	/* sting */
	PORTC |= (1 << LED_VERDE);	/* aprinde led */
}


void change_to_red()
{
	PORTD |= (1 << LED_ROSU);	/* aprinde led */
	PORTC &= ~(1 << LED_VERDE);	
}

	
	
int main(void) 
{

	initialize_ADC();
	
	init_sensor();
	
	initialize_timer();

	initialize_motors();
	
	/*
	 * setez viteza initiala a robotelului
	 * aceasta va creste treptat
	 */

	OCR1A = 40;
	OCR1B = 40;
		
	
	/* LEDURI */
	init_led();
	
	while(1)	{
		
		/* porneste cu o viteza in fata
		o creste treptat pana la valoarea maxima
		daca da de obstacol: opreste, da cu spatele, face dreapta
		merge inainte */
	
		
		if(ADC_get_conversion(ADC_INIT_CHANNEL) >= critical_DISTANCE) {
		/* simte obstacol */
			
			_delay_ms(100);
			
			if(ADC_get_conversion(ADC_INIT_CHANNEL) < critical_DISTANCE) {
			/* zgomot, nu avem obstacol */
				change_to_green();
				increase_speed();
				go_forward();
				continue;
			} 
			
			/* acum a vazut obstacol */
			change_to_red();
			
			stop_motors();
			
			go_back();
			_delay_ms(650);
			
			stop_motors(); /* vezi daca merge si fara */
			
			switch_right();
			
			stop_motors();
				
			_delay_ms(200); /* pauza intre citiri */	
		}
		else
		{
			/* nu a sesizat obstacol => merge inainte */
			change_to_green();
			increase_speed();
			go_forward();
		}
	}	
	return 0;
}